/* 
** DAVfs header file
** Copyright (C) 2001 SungHun Kim <hunkim@kebi.com>
**
** This library and program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**   
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA
*/

#ifndef __DAVFSD_H__
#define __DAVFSD_H__

#include <libdav.h>
#include <nsocket.h>
#include <propcache.h>

typedef int socket_t;

#define PORT 5555

#define CACHE_DIR "/tmp/"

#define MTAB "/etc/mtab"
#define TMP_MTAB "/etc/mtab.davfs"

#define PID_FILE "/tmp/davfsd.pid"

#ifndef INVALID_SOCKET
#define INVALID_SOCKET -1
#endif

/* df information */
#define DEFAULT_BLOCKS 1024
#define DEFAULT_BFREE  0

#define REMOVE_FROM_LIST(item, head, next)      \
   if ((item) == (head))                \
      head = (item)->next;              \
   else {                               \
      temp = head;                      \
      while (temp && (temp->next != (item))) \
         temp = temp->next;             \
      if (temp)                         \
         temp->next = (item)->next;     \
   }                                    \


struct descriptor_data {
    socket_t     descriptor;             /* file descriptor for socket           */
    nsocket      *ns;
    int          connected;              /* mode of 'connectedness'              */
    char         *output;
    void         *data;
    int          data_len;
    long         offset;                 /* read request */  
    long         length;
    dav_config   dc;                     /* dav_config */
    char         *orig_path;             /* original path */
    prop_cache   *pc[PROP_CACHE_SIZE];   /* prop cache buffer */
    struct descriptor_data *next;        /* link to next descriptor             */
};

#define DAV_MAXNAMELEN 256
struct dav_finfo {
    char            f_name[DAV_MAXNAMELEN];
    int             f_name_len; 
    int             f_isdir;
    off_t           f_size;
    time_t          f_atime;
    time_t          f_mtime;
    time_t          f_ctime;
};

/* Struct for get quota information */
struct dav_statfs {
    long f_blocks;
    long f_bfree;
};

/* client process function */
int client_attr(dav_config *dc, char *arg1, char *arg2);
int client_read(dav_config *dc, char *arg1, char *arg2);
int client_write(dav_config *dc, char *arg1, char *arg2);
int client_ls(dav_config *dc, char *arg1, char *arg2);
int client_cp(dav_config *dc, char *arg1, char *arg2);
int client_mv(dav_config *dc, char *arg1, char *arg2);
int client_unlink(dav_config *dc, char *arg1, char *arg2);
int client_mkdir(dav_config *dc, char *arg1, char *arg2);
int client_rmdir(dav_config *dc, char *arg1, char *arg2);
int client_close(dav_config *dc, char *arg1, char *arg2);
int client_create(dav_config *dc, char *arg1, char *arg2);
int client_open(dav_config *dc, char *arg1, char *arg2);
int client_badcmd(dav_config *dc, char *arg1, char *arg2);
int client_trunc(dav_config *dc, char *arg1, char *arg2);
int client_quit(dav_config *dc, char *arg1, char *arg2);
int client_setattr(dav_config *dc, char *arg1, char *arg2);
int client_statfs(dav_config *dc, char *arg1, char *arg2);
int client_attr_cache(dav_config *dc, struct dav_finfo *finfo);
int dav_mk_res(char *buf, int ret, int count);

/* uri */
char *reduce_path(char *);

/* hash list for local cache*/
int set_hash(long ino, int fp);
int get_hash(long ino, int *fp);
int add_hash(long ino, int wish, int fp, char *path);
int del_hash(long ino, int *wish, int *fp, char *path);

/* davfsd.c */
void log(const char *format, ...);

/* umntall.c */
int umount_all(void);

/* uri */
int unescape_uri(char *uri);
int esacpe_uri(char *, const char *);

#endif









