/* 
** Hashed linked list for PROPFIND
** Copyright (C) 2001 SungHun Kim <hunkim@kebi.com>
**
** This library and program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**   
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA
*/

#ifndef __PROPCACHE_H__
#define __PROPCACHE_H__

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/timeb.h>

#include <davfsd.h>
#include <libdav.h>

#define PROP_CACHE_SIZE 256
#define CACHE_TIMEOUT 60   /* sec */

/* data struct for cache */
typedef struct {
    struct dav_finfo *finfo;       /* file info */
    char             *path;        /* path */
    int              isdirty;      /* dirty flag */
    time_t           ctime;        /* cache time */
}prop_cache;

/* prototype */
int init_prop_cache(prop_cache **pc);
int free_prop_cache(prop_cache **pc);
int add_prop_cache(prop_cache **pc, char *path, struct dav_finfo *finfo);
int get_prop_cache(prop_cache **pc, char *path, struct dav_finfo *finfo);
int invalidate_prop_cache(prop_cache **pc, char *path);

#endif










