/*
**   URI reducer and processor
**   Copyright (C) 2001 SungHun Kim <hunkim@kebi.com>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
** MA 02111-1307, USA
**
*/

//#define TEST

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include <davfsd.h>
#include <libdav.h>

#ifdef TEST
void safe_free(void *ptr) {
    if(ptr) free(ptr);
    ptr = NULL;
}

char* safe_strdup(const char *s) {
    return strdup(s);
}

#define BUF_SIZE 256
#endif

void safe_free(void *ptr);

typedef struct one_path_t {
    char *path;
    struct one_path_t *prev, *next;
} one_path;

static int ind(char *s, char c) {
    register int x;

    for(x=0;s[x];x++)
        if(s[x] == c) return x;

    return -1;
}


int escape_uri(char *des, char *src) {
    while(*src) {
	if(!isalnum(*src) && ind("&$-_.+!*'(),:@@=~/?%", *src) == -1) {
	    *des++='%';
	    sprintf(des, "%02x", *src++);
	    des+=2;
	} else {
	    *des++=*src++;
	}
    }
    *des = 0;
    return 0;
}

/*
** reduce path
** 
** '//java/hunkim/../url/./..'  -> '/java/hunkim/'
** '//../../hunkim/java/./ '    -> '/java/hunkim/'
**
**
*/
char *reduce_path(char *path) {
    one_path *root, *f, *p;
    char buf[BUF_SIZE];
    char e_uri[BUF_SIZE];

    int slash = (*(path+strlen(path)-1)=='/');

    p = root = malloc(sizeof(one_path));
    memset(root, 0x00, sizeof(one_path));

    /* tokenize */
    p->path = strtok(path, "/\n\r");
    p->prev = NULL;

    /* nothing to reduce */
    if(p->path==NULL) {
	return safe_strdup("/");
    }

    /* make structure */
    while(p->path!=NULL) {
	char *tok = strtok(NULL, "/\n\r");
	if(tok==NULL) break;

	p->next = malloc(sizeof(one_path));
	memset(p->next, 0x00, sizeof(one_path));

	p->next->prev = p;
	p = p->next;
	p->path = tok;
    }
    
    /* walk thorough path */
    for(p=root; p; p=p->next) {
	if(!strcmp(p->path, "..")) {
	    p->path=NULL;
	    if(p->prev) p->prev->path=NULL;
	} 
	else if(!strcmp(p->path, ".")) 
	    p->path=NULL;
    }
    
    memset(buf, 0x00, sizeof(buf));
    
    /* make real path */
    for(p=root; p; p=p->next, safe_free(f)) {
	if(p->path)  {
	    strcat(buf, "/");
	    // escape uri
	    escape_uri(e_uri, p->path);
	    strcat(buf, e_uri);
	}
	f = p;
    }
    if(slash) strcat(buf, "/");
    return safe_strdup(buf);
}

static int hex_dec(char c) {
    if(isdigit(c)) 
	return (int)(c-'0');

    if(c>='A' && c<='F') 
	return (int)(c-'A'+10);

    if(c>='a' && c<='f') 
	return (int)(c-'a'+10);

    // error
    return 0;
}
 
static char x2c(char *x) {
    char ret=0;
    
    ret+=(hex_dec(*x))*16;
    ret+=hex_dec(*(x+1));

    return ret;
}

/*
** unescape url
**
** change uri 
*/
int unescape_uri(char *uri) {
    char *x, *y;
    int ret = 0;

    for(x=y=uri;*x;x++, y++) {
	if(*x=='%') {
	    //just ignore %
	    if(!isxdigit(*(x+1)) || !isxdigit(*(x+2))) 
		x++, ret++;
	    else *y = x2c(x+1);
	    
	    x+=2;
	} else { /* no escape character */
	    *y=*x;
	}
    }
    
    *y = 0;
    return ret;
}
#ifdef TEST
int main() {
    char buf[BUF_SIZE];
    char *ptr;

    while(1) {
	printf("input path:");
	gets(buf);
	
	ptr = reduce_path(buf);
	puts(ptr);
	safe_free(ptr);
    }
}
#endif





























